import h5py
import numpy as np
import ipyvolume as ipv
from matplotlib import cm


def instrument_view(filename,
           id_path="/entry/event_data/event_id",
           colormap="viridis",
           log=False,
           size=1):
    """
    Make a 3D instrument view using ipyvolume
    """

    with h5py.File(filename, "r") as f:

        ids = np.array(f[id_path][...], dtype=np.int32, copy=True)
        # tofs = np.array(f[tof_path][...], dtype=np.float64, copy=True) / 1.0e3
        x = np.array(f["/entry/instrument/detector_1/x_pixel_offset"][...],
                     dtype=np.float64, copy=True)
        y = np.array(f["/entry/instrument/detector_1/y_pixel_offset"][...],
                     dtype=np.float64, copy=True)


        # location = float(f["/entry/instrument/detector_1/transformations/location"][...])
        # loc_vec = np.array(f["/entry/instrument/detector_1/transformations/location"].attrs["vector"][...], dtype=np.float64, copy=True)
        # rotation = float(f["/entry/instrument/detector_1/transformations/orientation"][...])
        # rot_vec = np.array(f["/entry/instrument/detector_1/transformations/orientation"].attrs["vector"][...], dtype=np.float64, copy=True)

    nx, ny = np.shape(x)
    a, edges = np.histogram(ids, bins=np.arange(nx * ny + 1))
    a = a.reshape(nx, ny)
    if log:
        with np.errstate(divide="ignore", invalid="ignore"):
            a = np.log10(a)

    # x = x.flatten()
    # y = y.flatten()
    # a = a.flatten()

    z = np.zeros_like(x)

    # # Apply rotation
    # # TODO: this currently only works for rotation around y axis
    # # Need to generalise this with rotation matrix
    # rotation *= np.pi / 180.0
    # x = x * np.cos(rotation)
    # z = x * np.sin(rotation)
    # # Apply translation
    # x += location * loc_vec[0]
    # y += location * loc_vec[1]
    # z += location * loc_vec[2]

    # x *= 100.
    # y *= 100.
    # z *= 100.
    

    norm = cm.colors.Normalize(vmax=a.max(), vmin=a.min())
    colormap = cm.viridis
    color = colormap(norm(a.flatten()))

    # t = np.linspace(0.0, 7.2e4, nbins + 1)
    # z, xe, ye = np.histogram2d(ids, tofs, bins=[np.arange(nx * ny + 1), t])
    # z = np.transpose(z.reshape(nx, ny, nbins), axes=[2, 1, 0])
    # ipv.quickvolshow(z,
    #     extent=[[x[0, 0]*100.0, x[0, -1]*100.0],
    #             [y[0, 0]*100.0, y[-1, 0]*100.0],
    #             [t[0], t[-1]]])
    # ipv.pylab.xlabel("x [cm]")
    # ipv.pylab.ylabel("y [cm]")
    # ipv.pylab.zlabel("Tof [us]")
    # print(x)
    # print(y)
    # print(z)
    
    ipv.figure()
    x1, y1 = np.meshgrid([-1, 1], [-1, 1])
    w1 = ipv.plot_wireframe(x1, y1, np.ones_like(x1) * (-1.0), color="black")
    w2 = ipv.plot_wireframe(x1, y1, np.ones_like(x1), color="black")

    surf = ipv.plot_surface(x, y, z, color=color[...,:3])


    ipv.pylab.scatter([-1, -0.5], [-1, -0.5], [-1, -0.5], size=3, marker="sphere", color='r')
    ipv.show()

    return



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider
from .load import load


class Slicer(object):
    def __init__(self, fig, ax, X, tof, dt=0.0, vmin=None, vmax=None,
                 extent=None):
        self.fig = fig
        self.ax = ax
        self.ax.set_xlabel("x position [m]")
        self.ax.set_ylabel("y position [m]")

        self.X = X
        self.tof = tof
        self.dt = dt
        rows, cols, self.slices = X.shape
        self.ind = 0

        self.im = ax.imshow(self.X[:, :, self.ind], origin="lower",
                            aspect="equal", interpolation="none",
                            vmin=vmin, vmax=vmax, extent=extent)
        self.cb = plt.colorbar(self.im, ax=self.ax)
        self.cb.ax.set_ylabel("Counts")
        self.fig.canvas.mpl_connect('scroll_event', self.onscroll)

        # Add mpl slider widget
        self.ax_slider = self.fig.add_axes([0.23, 0.02, 0.56, 0.04])
        self.slider = Slider(self.ax_slider,
            'Tof [us]',
            tof.min(),
            tof.max(),
            valinit=tof.min())
        # Connect slider to function
        self.slider.on_changed(self.update)

    def onscroll(self, event):
        """
        Allow moving the slider with the mouse wheel
        """
        if event.button == 'up':
            self.ind = np.clip(self.ind + 1, 0, self.slices - 1)
        else:
            self.ind = np.clip(self.ind - 1, 0, self.slices - 1)
        self.slider.set_val(self.ind * self.dt)

    def update(self, val=None):
        """
        Update the image with new data according to slider value
        """
        self.ind = int(round(val/self.dt))
        self.im.set_data(self.X[:, :, self.ind])
        # self.ax.set_title("slice {}".format(self.ind))



def slicer(filename, colormap="viridis", vmin=None, vmax=None, log=False,
           nbins=256, transpose=False):
    """
    Make a 2D image viewer with a slider to navigate in the Tof dimension.
    You can also scroll with the mouse wheel to change the slider position.
    """

    data = load(filename, ids=True, tofs=True)

    nx, ny = np.shape(data.x)
    t = np.linspace(0.0, 7.2e4, nbins + 1)
    z, xe, ye = np.histogram2d(data.ids, data.tofs/1.0e3,
                               bins=[np.arange(nx * ny + 1), t])
    z = z.reshape(nx, ny, nbins)
    # Transpose should be True for old December 2018 files
    if transpose:
        z = np.transpose(z, axes=[1, 0, 2])
    if log:
        with np.errstate(divide="ignore", invalid="ignore"):
            z = np.log10(z)

    fig, ax = plt.subplots(1, 1)
    ax.set_title(filename.split("/")[-1])
    sl = Slicer(fig, ax, z, 0.5*(t[1:] + t[:-1]), dt=t[1]-t[0],
                vmin=vmin, vmax=vmax, extent=[data.x[0, 0], data.x[0, -1],
                                              data.y[0, 0], data.y[-1, 0]])
    plt.show()
    return sl



import numpy as np
import matplotlib.pyplot as plt
from .load import load


def image(filename, colormap="viridis", vmin=None, vmax=None, log=False,
          save=None):
    """
    Make a 2D image of the detector counts
    """

    data = load(filename, ids=True)

    nx, ny = np.shape(data.x)
    z, edges = np.histogram(data.ids, bins=np.arange(nx * ny + 1))
    z = z.reshape(nx, ny)
    if log:
        with np.errstate(divide="ignore", invalid="ignore"):
            z = np.log10(z)

    fig = plt.figure()
    ax = fig.add_subplot(111)
    im = ax.imshow(z, origin="lower", aspect="equal", interpolation="none",
                   vmin=vmin, vmax=vmax, extent=[data.x[0, 0], data.x[0, -1],
                                                 data.y[0, 0], data.y[-1, 0]])
    cb = plt.colorbar(im, ax=ax)
    cb.ax.set_ylabel("Counts")
    ax.set_xlabel("x position [m]")
    ax.set_ylabel("y position [m]")
    ax.set_title(filename.split("/")[-1])
    if save is not None:
        extension = "." + f.split(".")[-1]
        fig.savefig(f.replace(extension, ".pdf"), bbox_inches="tight")
    else:
        plt.show()



import numpy as np
from .load import load


def volume(filename, colormap="viridis", nbins=256):
    """
    Make a 3D volume rendering using ipyvolume
    """

    import ipyvolume as ipv

    data = load(filename, ids=True, tofs=True)

    nx, ny = np.shape(data.x)
    t = np.linspace(0.0, 7.2e4, nbins + 1)
    z, xe, ye = np.histogram2d(data.ids, data.tofs/1.0e3,
                               bins=[np.arange(nx * ny + 1), t])
    z = np.transpose(z.reshape(nx, ny, nbins), axes=[2, 1, 0])
    ipv.quickvolshow(z,
        extent=[[data.x[0, 0]*100.0, data.x[0, -1]*100.0],
                [data.y[0, 0]*100.0, data.y[-1, 0]*100.0],
                [t[0], t[-1]]])
    ipv.pylab.xlabel("x [cm]")
    ipv.pylab.ylabel("y [cm]")
    ipv.pylab.zlabel("Tof [us]")
    ipv.show()

    return
